package win32neshto.j2mesafemod;

import javax.microedition.lcdui.*;
/**
 *
 * @author Win32.Neshto
 */
public class GeneratePasswordOptionsForm extends Form implements CommandListener {
    private J2MESafe _app;
    private TextField lengthField;
    private ChoiceGroup charsetsChoice;
    private TextField specialCharsField;
    private Command okCommand = new Command(Str.OK, Command.OK, 1);
    private Command backCommand = new Command(Str.BACK, Command.BACK, 2);
    private SetTextInterface d;
    
    public GeneratePasswordOptionsForm(J2MESafe app, SetTextInterface d) {
        super(Str.PASSWORD_GENERATION_OPTIONS);
        this.d = d;
        _app = app;
        append(Str.MIN_LENGTH1+String.valueOf(PasswordGenerator.MIN_LENGTH)+Str.MIN_LENGTH2);
        lengthField = new TextField(Str.LENGTH, "8", 3, TextField.NUMERIC);
        append(lengthField);
        charsetsChoice = new ChoiceGroup(Str.CHARSETS, Choice.MULTIPLE);
        charsetsChoice.append(Str.EN_SMALL, null);
        charsetsChoice.append(Str.EN_CAP, null);
        charsetsChoice.append(Str.DIGITS, null);
        charsetsChoice.append(Str.SPECIAL_CHARS, null);
        charsetsChoice.setSelectedFlags(new boolean[] {true, true, true, true});
        append(charsetsChoice);
        String defaultSpecialChars = "";
        for(int i = 0; i<PasswordGenerator.DEFAULT_SPECIAL_CHARS.length; i++) {
            defaultSpecialChars += String.valueOf(PasswordGenerator.DEFAULT_SPECIAL_CHARS[i]);
        }
        specialCharsField = new TextField(Str.SPECIAL_CHARS, defaultSpecialChars,
                100, TextField.NON_PREDICTIVE);
        append(specialCharsField);
        addCommand(okCommand);
        addCommand(backCommand);
        setCommandListener(this);
    }
    
    //null if invalid options
    public PasswordGenerator getGenerator() {
        int charset = 0;
        boolean[] selectedFlags = new boolean[4];
        charsetsChoice.getSelectedFlags(selectedFlags);
        if(selectedFlags[0]) charset |= PasswordGenerator.SMALL;
        if(selectedFlags[1]) charset |= PasswordGenerator.CAPS;
        if(selectedFlags[2]) charset |= PasswordGenerator.DIGITS;
        if(selectedFlags[3]) charset |= PasswordGenerator.SPECIAL;
        if(charset==0) {
            return null;
        }
        int length = 0;
        try {
            length = Integer.parseInt(lengthField.getString());
            if((length<PasswordGenerator.MIN_LENGTH)||(length>Consts.MAX_TEXT_LENGTH)) {
                return null;
            }
        } catch(Exception exc) {
            return null;
        }
        char[] specialChars = null;
        if(selectedFlags[3]) {
            String origText = specialCharsField.getString();
            String resText = "";
            int origLength = origText.length();
            if(origLength==0) {
                return null;
            }
            int resLength = 0;
            loop:for(int i = 0; i<origLength; i++) {
                char ch = origText.charAt(i);
                switch(ch) {
                    case ' ':
                    case '\t':
                    case '\r':
                    case '\n':
                        return null;
                }
                for(int j = 0; j<resLength; j++) {
                    if(ch==resText.charAt(j)) {
                        continue loop;
                    }
                }
                resText += String.valueOf(ch);
                resLength++;
            }
            if(resLength==0) {
                return null;
            }
            specialChars = resText.toCharArray();
        }
        try {
            if(specialChars==null) {
                return new PasswordGenerator(charset, length);
            } else {
                return new PasswordGenerator(charset, length, specialChars);
            }
        } catch(RuntimeException exc) {
            return null;
        }
    }

    public void commandAction(Command c, Displayable _d) {
        if(c==okCommand) {
            PasswordGenerator gen = getGenerator();
            if(gen==null) {
                Alert a = new Alert(Str.ERROR, Str.INVALID_OPTIONS, null, AlertType.ERROR);
                a.addCommand(okCommand);
                _app.getDisplay().setCurrent(a, this);
                return;
            }
            _app.getDisplay().setCurrent(new PasswordGenerateForm(gen, _app, d));
        } else if(c==backCommand) {
            _app.getDisplay().setCurrent(d.getDisplayable());
        }
    }
}
