package win32neshto.j2mesafemod;

import javax.microedition.rms.RecordStore;

import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.SHA1Digest;

/**
 * saves a SHA1 digest of the users password in persistent storage.
 */
public class PasswordMgr {
  byte[] digest;
  public static final String STORE = "j2mesafe.digest";

  public PasswordMgr() {
    loadDigest();
  }

  /**
   * compares the digest of the password with the stored digest. returns true if
   * they are equal
   */
  public boolean match(String password) {
    if (digest == null)
      return false; // not initialized
    byte[] test = createDigest(password);
    if (digest.length != test.length)
      return false;
    for (int i = 0; i < test.length; i++)
      if (digest[i] != test[i])
        return false;
    return true;
  }

  /**
   * sets the initial password. Computes the digest from password and stores it
   * in persistent storage.
   */
  public void initialize(String password) {
    if (password == null) {
      try {
        RecordStore.deleteRecordStore(STORE);
      } catch (Exception e) {
        e.printStackTrace();
      }
      digest = null;
    } else {
      this.digest = createDigest(password);
      saveDigest();
    }
  }

  public boolean isInitialized() {
    return digest != null;
  }

  private byte[] createDigest(String password) {
    Digest algo = new SHA1Digest();
    byte[] bytes = password.getBytes();
    algo.update(bytes, 0, bytes.length);
    byte[] digest = new byte[algo.getDigestSize()];
    algo.doFinal(digest, 0);
    return digest;
  }

  private void loadDigest() {
    RecordStore store = null;
    try {
      store = RecordStore.openRecordStore(STORE, true);
      if (store.getNumRecords() == 0)
        return;
      digest = store.getRecord(1);
    } catch (Exception e) {
      e.printStackTrace();
      return;
    } finally {
      try {
        if (store != null)
          store.closeRecordStore();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  private void saveDigest() {
    RecordStore store = null;
    try {
      store = RecordStore.openRecordStore(STORE, true);
      if (store.getNumRecords() == 0)
        store.addRecord(digest, 0, digest.length);
      else
        store.setRecord(1, digest, 0, digest.length);
    } catch (Exception e) {
      e.printStackTrace();
      return;
    } finally {
      if (store != null) {
        try {
          store.closeRecordStore();
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    }
  }
}