package win32neshto.j2mesafemod;

import org.bouncycastle.crypto.BufferedBlockCipher;
import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.engines.TwofishEngine;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.paddings.PaddedBufferedBlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;

/**
 * encrypts data using bouncycastle twofish engine
 */
public class TwofishCryptMgr implements CryptMgr {
  BufferedBlockCipher engine;
  KeyParameter key;

  public void initialize(String passcode) {
    // create a 16 byte key from passcode
    byte[] kb = new byte[16];
    for (int i = 0; i < kb.length; i++)
      kb[i] = (byte) (i + 20);
    byte[] pb = passcode.getBytes();
    for (int ki = 0, pi = 0; pi < pb.length; pi++) {
      kb[ki] += pb[pi];
      ki = (ki + 1) % 16;
    }
    // initialize key and engine
    key = new KeyParameter(kb);
    engine = new PaddedBufferedBlockCipher(new CBCBlockCipher(new TwofishEngine()));
  }

  public byte[] encrypt(byte[] bytes) {
    engine.init(true, key);
    return doCipher(bytes);
  }

  public byte[] decrypt(byte[] bytes) {
    engine.init(false, key);
    return doCipher(bytes);
  }

  private byte[] doCipher(byte[] input) {
    byte[] output = new byte[engine.getOutputSize(input.length)];
    int len = engine.processBytes(input, 0, input.length, output, 0);
    try {
      engine.doFinal(output, len);
    } catch (CryptoException e) {
      e.printStackTrace();
      return null;
    }
    return output;
  }
}