package win32neshto.j2mesafemod;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class Entry {
  int id;
  String label;
  String body;

  public Entry(int id) {
    this.id = id;
  }

  public Entry(int id, byte[] bytes) {
    this(id);
    setBytes(bytes);
  }
  
  public Entry(String label, String body) {
    this(0);  // invalid recordId
    this.label = label;
    this.body = body;
  }

  /**
   * returns the state of this entry.
   */
  public byte[] getBytes() {
    try {
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      DataOutputStream dos = new DataOutputStream(bos);
      dos.writeUTF(label);
      dos.writeUTF(body);
      dos.close();
      return bos.toByteArray();
    } catch (IOException e) {
      e.printStackTrace();
      return null;
    }
  }

  /**
   * sets the state of this entry
   */
  public void setBytes(byte[] bytes) {
    try {
      ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
      DataInputStream dis = new DataInputStream(bis);
      label = dis.readUTF();
      body = dis.readUTF();
      dis.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * @return Returns the id.
   */
  public int getId() {
    return id;
  }

  /**
   * @return Returns the body.
   */
  public String getBody() {
    return body;
  }

  /**
   * @param body
   *          The body to set.
   */
  public void setBody(String body) {
    this.body = body;
  }

  /**
   * @return Returns the label.
   */
  public String getLabel() {
    return label;
  }

  /**
   * @param label
   *          The label to set.
   */
  public void setLabel(String label) {
    this.label = label;
  }

  /**
   * @param id
   *          The id to set.
   */
  public void setId(int id) {
    this.id = id;
  }
  
  public static int compare(Entry e1, Entry e2) {
      return e1.label.compareTo(e2.label);
  }
  
  
  
}