package win32neshto.j2mesafemod;

import java.util.Vector;
import javax.microedition.rms.*;
import java.io.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.Digest;

/**
 * holds a vector of Entry instances for the List
 */
public class EntryMgr {
  static final String STORE = "j2mesafe.data";
  static final String BACKUP_PATH_STORE = "j2mesafe.backuppath";
  private CryptMgr cryptMgr;
  private Vector entries = new Vector();

  public EntryMgr(CryptMgr cryptMgr) {
    this.cryptMgr = cryptMgr;
  }

  public void deleteAll() {
    try {
      RecordStore.deleteRecordStore(STORE);
    } catch (RecordStoreNotFoundException e) {
      // ignore
    } catch (Exception e) {
      e.printStackTrace();
    }
    entries.removeAllElements();
  }

  public Entry createEntry(String label, String body) {
    Entry entry = new Entry(label, body);
    entries.addElement(entry);
    saveEntry(entry);
    return entry;
  }

  public void deleteEntry(Entry entry) {
    RecordStore rs = null;
    try {
      rs = RecordStore.openRecordStore(STORE, true);
      rs.deleteRecord(entry.getId());
      entries.removeElement(entry);
    } catch (RecordStoreException e) {
      e.printStackTrace();
    } finally {
      try {
        rs.closeRecordStore();
      } catch (RecordStoreException e) {
        e.printStackTrace();
      }
    }
  }

  public void changeEntry(Entry entry) {
    saveEntry(entry);
  }

  private void saveEntry(Entry entry) {
    RecordStore rs = null;
    try {
      rs = RecordStore.openRecordStore(STORE, true);
      byte[] b = entry.getBytes();
      b = cryptMgr.encrypt(b);
      int id = entry.getId();
      if (id == 0) {
        // new entry
        id = rs.addRecord(b, 0, b.length);
        entry.setId(id);
      } else {
        // existing entry
        rs.setRecord(id, b, 0, b.length);
      }
    } catch (RecordStoreException e) {
      e.printStackTrace();
    } finally {
      try {
        rs.closeRecordStore();
      } catch (RecordStoreException e) {
        e.printStackTrace();
      }
    }
  }

  /**
   * loads all records from store. CryptMgr must be initialized
   * to decrypt the entries
   */
  public void initialize() {
    RecordStore rs = null;
    RecordEnumeration en = null;
    entries.removeAllElements();
    try {
      rs = RecordStore.openRecordStore(STORE, true);
      en = rs.enumerateRecords(null, null, false);
      while (en.hasNextElement()) {
        int id = en.nextRecordId();
        byte[] bytes = rs.getRecord(id);
        bytes = cryptMgr.decrypt(bytes);
        Entry entry = new Entry(id, bytes);
        entries.addElement(entry);
      }
    } catch (RecordStoreException e) {
      e.printStackTrace();
    } finally {
      if (en != null)
        en.destroy();
      try {
        if (rs != null)
          rs.closeRecordStore();
      } catch (RecordStoreException e1) {
        e1.printStackTrace();
      }
    }
  }
  
  public void sort() {
      int count = entries.size();
      for(int i = 0; i<count-1; i++) {
          int k = i;
          for(int j = i+1; j<count; j++) {
              if(Entry.compare((Entry)entries.elementAt(j), (Entry)entries.elementAt(k))<0) {
                  k = j;
              }
          }
          if(i!=k) {
              Object t = entries.elementAt(i);
              entries.setElementAt(entries.elementAt(k), i);
              entries.setElementAt(t, k);
          }
      }
  }

  public Vector getEntries() {
    return entries;
  }
  
  /**
   * @author Win32.Neshto
   */
  public void backup(String path, byte[] digest) throws IOException {
      FileConnection fc = (FileConnection)Connector.open(path, Connector.READ_WRITE);
      try {
          if(fc.exists()) {
              fc.delete();
          }
          fc.create();
          DataOutputStream stream = fc.openDataOutputStream();
          try {
              stream.writeInt(digest.length);
              stream.write(digest);
              int count = entries.size();
              for(int i = 0; i<count; i++) {
                  Entry entry = (Entry)entries.elementAt(i);
                  entry.setId(-1);
                  byte[] encrypted = cryptMgr.encrypt(entry.getBytes());
                  stream.writeInt(encrypted.length);
                  stream.write(encrypted);
              }
              stream.writeInt(-1);
          } finally {
              stream.close();
          }
      } finally {
          fc.close();
      }
  }
  
  /**
   * @author Win32.Neshto
   */
  //call instead of initialize. false is invalid password. cryptMgr must have the same password
  public boolean restore(String path, String password) throws Exception {
      FileConnection fc = (FileConnection)Connector.open(path, Connector.READ);
      try {
          if(!fc.exists()) {
              throw new IOException();
          }
          DataInputStream stream = fc.openDataInputStream();
          try {
              byte[] digest = new byte[stream.readInt()];
              stream.read(digest);
              Digest algo = new SHA1Digest();
              algo.update(password.getBytes(), 0, password.getBytes().length);
              byte[] passDigest = new byte[algo.getDigestSize()];
              algo.doFinal(passDigest, 0);
              if(digest.length!=passDigest.length) {
                  return false;
              }
              for(int i = 0; i<digest.length; i++) {
                  if(digest[i]!=passDigest[i]) {
                      return false;
                  }
              }
              entries.removeAllElements();
              try {
                  RecordStore.deleteRecordStore(STORE);
              } catch(RecordStoreNotFoundException exc) {
                  //ignore
              }
              RecordStore rs = RecordStore.openRecordStore(STORE, true);
              try {
                  for(int len = stream.readInt(); len>=0; len = stream.readInt()) {
                      byte[] bytes = new byte[len];
                      stream.read(bytes);
                      int id = rs.addRecord(bytes, 0, bytes.length);
                      entries.addElement(new Entry(id, cryptMgr.decrypt(bytes)));
                  }
              } finally {
                  rs.closeRecordStore();
              }
          } finally {
              stream.close();
          }
      } finally {
          fc.close();
      }
      return true;
  }
  
  public static String getBackupPath() {
      try {
          RecordStore rs = RecordStore.openRecordStore(BACKUP_PATH_STORE, true);
          try {
              RecordEnumeration re = rs.enumerateRecords(null, null, false);
              try {
                  if(!re.hasNextElement()) {
                      return "";
                  } else {
                      DataInputStream stream = 
                              new DataInputStream(new ByteArrayInputStream(re.nextRecord()));
                      try {
                          return stream.readUTF();
                      } finally {
                          stream.close();
                      }
                  }
              } finally {
                  re.destroy();
              }
          } finally {
              rs.closeRecordStore();
          }
      } catch(Exception exc) {
          return "";
      }
  }
}