package win32neshto.j2mesafemod;

import java.io.*;
import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;

/**
 *
 * @author Win32.Neshto
 */
public class FileBrowser implements CommandListener {
    private List list;
    private TextBox fileNameTextBox = null;
    private Command okCommand;
    private Command cancelCommand;
    private Command upCommand;
    private Command newFileHereCommand;
    private Command createFolderCommand;
    private Path current;
    private SetTextInterface sti;
    private Alert err;
    private J2MESafe app;
    private Vector dirs;
    private int dirsCount;
    private Vector files;
    private int filesCount;
    private boolean newFileFlag;
    private Image IMG_DIR;
    private Image IMG_FILE;
    
   
    public FileBrowser(SetTextInterface d, J2MESafe app, boolean save) {
        sti = d;
        this.app = app;
        list = new List("", List.IMPLICIT);
        okCommand = new Command(Str.OK, Command.OK, 10);
        cancelCommand = new Command(Str.CANCEL, Command.CANCEL, 20);
        upCommand = new Command(Str.UP, Command.SCREEN, 20);
        newFileHereCommand = new Command(Str.NEW_FILE, Command.SCREEN, 30);
        createFolderCommand = new Command(Str.NEW_FOLDER, Command.SCREEN, 40);
        list.addCommand(okCommand);
        list.addCommand(cancelCommand);
        list.addCommand(upCommand);
        if(save) {
            list.addCommand(newFileHereCommand);
            list.addCommand(createFolderCommand);
            fileNameTextBox = new TextBox("", "", 50, TextField.NON_PREDICTIVE);
            fileNameTextBox.addCommand(okCommand);
            fileNameTextBox.addCommand(cancelCommand);
            fileNameTextBox.setCommandListener(this);
        }
        list.setSelectCommand(okCommand);
        list.setCommandListener(this);
        err = new Alert(Str.ERROR, "", null, AlertType.ERROR);
        err.addCommand(new Command(Str.OK, Command.OK, 1));
        err.setTimeout(Alert.FOREVER);
        try {
            current = Path.getRoot();
            IMG_DIR = Image.createImage("/dir.png");
            IMG_FILE = Image.createImage("/file.png");
            showCurrDir();
        } catch(Exception exc) {
            showError(Str.FILE_BROWSER_INIT_ERROR);
        }
    }
    
    public void show() {
        app.getDisplay().setCurrent(list);
    }
    
    private void showError(String message) {
        err.setString(message);
        app.getDisplay().setCurrent(err, sti.getDisplayable());
    }
    
    private void showCurrDir() {
        list.setTitle(current.getName());
        list.deleteAll();
        dirs = new Vector();
        files = new Vector();
        try {
            current.list(dirs, files);
        } catch(Exception exc) {
            showError(Str.FILE_BROWSER_ERROR);
        }
        dirsCount = dirs.size();
        for(int i = 0; i<dirsCount; i++) {
            list.append(((Path)dirs.elementAt(i)).getName(), IMG_DIR);
        }
        filesCount = files.size();
        for(int i = 0; i<filesCount; i++) {
            list.append(((Path)files.elementAt(i)).getName(), IMG_FILE);
        }
    }
    
    public void commandAction(Command c, Displayable d) {
        if(d==list) {
            if(c==okCommand) {
                int index = list.getSelectedIndex();
                if(index<0) {
                    return;
                } else if(index<dirsCount) {
                    current = (Path)dirs.elementAt(index);
                    showCurrDir();
                } else {
                    index -= dirsCount;
                    Path file = (Path)files.elementAt(index);
                    sti.setText(file.toString());
                    app.getDisplay().setCurrent(sti.getDisplayable());
                }
            } else if(c==cancelCommand) {
                app.getDisplay().setCurrent(sti.getDisplayable());
            } else if(c==upCommand) {
                if(!current.isRoot()) {
                    current = current.getParentDir();
                    showCurrDir();
                }
            } else if(c==newFileHereCommand) {
                newFileFlag = true;
                app.getDisplay().setCurrent(fileNameTextBox);
            } else if(c==createFolderCommand) {
                newFileFlag = false;
                app.getDisplay().setCurrent(fileNameTextBox);
            }
        } else if(d==fileNameTextBox) {
            if(c==okCommand) {
                String name = fileNameTextBox.getString();
                if(name.length()==0) {
                    return;
                }
                if(fileWithThisNameExists(name)) {
                    err.setString(Str.FILE_NAME_CONFLICT);
                    app.getDisplay().setCurrent(err, fileNameTextBox);
                    return;
                }
                if(newFileFlag) {
                    sti.setText(current.toString()+fileNameTextBox.getString());
                    app.getDisplay().setCurrent(sti.getDisplayable());
                } else {
                    try {
                        String newPath = current.toString()+name;
                        FileConnection newDir = 
                                (FileConnection)Connector.open(Consts.FILE_URL_PREFIX+newPath);
                        try {
                            newDir.mkdir();
                        } finally {
                            newDir.close();
                        }
                        current = new Path(newPath+Consts.DIR_SEPARATOR);
                        showCurrDir();
                        app.getDisplay().setCurrent(list);
                    } catch(Exception exc) {
                        err.setString(Str.UNABLE_TO_CREATE_FOLDER);
                        app.getDisplay().setCurrent(err, list);
                    }
                }
            } else if(c==cancelCommand) {
                app.getDisplay().setCurrent(list);
            }
        }
    }
    
    private boolean fileWithThisNameExists(String name) {
        Enumeration e = files.elements();
        while(e.hasMoreElements()) {
            if(((Path)e.nextElement()).getName().equals(name)) {
                return true;
            }
        }
        e = dirs.elements();
        while(e.hasMoreElements()) {
            if(((Path)e.nextElement()).getName().equals(name)) {
                return true;
            }
        }
        return false;
    }
}
