package win32neshto.j2mesafemod;

import java.util.Enumeration;
import java.io.*;
import java.util.Vector;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;

public class EntryList extends List implements CommandListener {
  private EntryMgr entryMgr;
  private J2MESafe _app;
  Command showCommand = new Command(Str.SHOW, Command.SCREEN, 1);
  Command createCommand = new Command(Str.NEW_ENTRY, Command.SCREEN, 1);
  Command changeCommand = new Command(Str.CHANGE, Command.SCREEN, 1);
  Command deleteCommand = new Command(Str.DELETE, Command.SCREEN, 1);
  Command backupCommand = new Command(Str.BACKUP, Command.SCREEN, 1);
  Command aboutCommand = new Command(Str.ABOUT, Command.SCREEN, 1);

  public EntryList(J2MESafe app) {
    super(Str.ENTRIES, List.IMPLICIT);
    this.entryMgr = app.entryMgr;
    this._app = app;
    addCommand(app.exitCommand);
    addCommand(createCommand);
    addCommand(changeCommand);
    addCommand(deleteCommand);
    addCommand(backupCommand);
    addCommand(aboutCommand);
    setCommandListener(this);
  }

  public void commandAction(Command c, Displayable d) {
    if (c == createCommand) {
      CreateEntryForm cef = new CreateEntryForm(_app);
      _app.getDisplay().setCurrent(cef);
    } else if (c == changeCommand) {
      Entry entry = getSelectedEntry();
      if (entry == null)
        return;
      Displayable screen = new ChangeEntryForm(_app, entry);
      _app.getDisplay().setCurrent(screen);
    } else if (c == deleteCommand) {
      Entry entry = getSelectedEntry();
      if (entry == null)
        return;
      Displayable screen = new DeleteEntryForm(_app, entry);
      _app.getDisplay().setCurrent(screen);
    } else if (c == showCommand || c == List.SELECT_COMMAND) {
      Entry entry = getSelectedEntry();
      if (entry == null)
        return;
      Displayable screen = new ShowEntryDetailForm(_app, entry);
      _app.getDisplay().setCurrent(screen);
    } else if(c == backupCommand) {
        final TextBox pathBox = new TextBox(Str.BACKUP_FILE_NAME, EntryMgr.getBackupPath(),
                1000, TextField.ANY);
        final Command okCommand = new Command(Str.OK, Command.OK, 1);
        final Command cancelCommand = new Command(Str.BACK, Command.BACK, 2);
        final Command browseCommand = new Command(Str.BROWSE, Command.SCREEN, 3);        
        pathBox.addCommand(okCommand);
        pathBox.addCommand(cancelCommand);
        pathBox.addCommand(browseCommand);
        pathBox.setCommandListener(new CommandListener() {
                public void commandAction(Command c, Displayable d) {
                    if(c==okCommand) {
                        String text = pathBox.getString();
                        if(text.length()==0) {
                            return;
                        }
                        String path = Consts.FILE_URL_PREFIX+text;
                        try {
                            byte[] digest = _app.passwordMgr.digest;
                            entryMgr.backup(path, digest);
                            Alert a1 = new Alert(Str.INFO, Str.BACKUP_SAVED,
                                    null, AlertType.INFO);
                            a1.addCommand(okCommand);
                            _app.getDisplay().setCurrent(a1, EntryList.this);
                        } catch(Exception exc) {
                            String atext =
                                (exc instanceof IOException)?Str.FILE_ERROR:Str.OTHER_ERROR;
                            Alert a1 = new Alert(Str.ERROR, Str.BACKUP_ERROR,
                                    null, AlertType.ERROR);
                            a1.addCommand(okCommand);
                            a1.setTimeout(Alert.FOREVER);
                            _app.getDisplay().setCurrent(a1, EntryList.this);
                        }
                        try {
                          RecordStore rs1 = RecordStore.openRecordStore(
                                  EntryMgr.BACKUP_PATH_STORE, true);
                          try {
                              RecordEnumeration re1 = rs1.enumerateRecords(null, null, false);
                              try {
                                  ByteArrayOutputStream baos = new ByteArrayOutputStream();
                                  DataOutputStream dos = new DataOutputStream(baos);
                                  try {
                                      dos.writeUTF(text);
                                  } finally {
                                      dos.close();
                                  }
                                  byte[] bytes1 = baos.toByteArray();
                                  if(re1.hasNextElement()) {
                                      rs1.setRecord(re1.nextRecordId(), bytes1, 0, bytes1.length);
                                  } else {
                                      rs1.addRecord(bytes1, 0, bytes1.length);
                                  }
                              } finally {
                                  re1.destroy();
                              }
                          } finally {
                              rs1.closeRecordStore();
                          }
                          } catch(Exception exc) {
                              //ignore
                          }                       
                    } else if(c==cancelCommand) {
                        _app.getDisplay().setCurrent(EntryList.this);
                    } else if(c==browseCommand) {
                        SetTextInterface sti = new SetTextInterface() {
                            public Displayable getDisplayable() {
                                return pathBox;
                            }

                            public void setText(String text) {
                                pathBox.setString(text);
                            }
                        };
                        FileBrowser fb = new FileBrowser(sti, _app, true);
                        fb.show();
                    }
                }
            });
        Alert a = new Alert(Str.INFO, Str.BACKUP_WARNING, null, AlertType.INFO);
        a.addCommand(okCommand);
        a.setTimeout(Alert.FOREVER);
        _app.getDisplay().setCurrent(a, pathBox);
    } else if(c==aboutCommand) {
        AboutForm af = new AboutForm(_app);
        af.show();
    } else {
      _app.commandAction(c, d);
    }
  }

  /**
   * called exactly once before the list is shown. Loads all entries from
   * entryMgr
   */
  public void initialize() {
    super.deleteAll();
    Enumeration en = entryMgr.getEntries().elements();
    while (en.hasMoreElements()) {
      Entry e = (Entry) en.nextElement();
      super.append(e.getLabel(), null);
    }
  }
  
  public void sort() {
      Entry selected_entry = getSelectedEntry();
      entryMgr.sort();
      initialize();
      if(selected_entry!=null) {
          Vector entries = entryMgr.getEntries();
          int count = entries.size();
          for(int i = 0; i<count; i++) {
              if(entries.elementAt(i)==selected_entry) {
                  this.setSelectedIndex(i, true);
                  break;
              }
          }
      }
  }

  public void deleteAll() {
    entryMgr.deleteAll();
    while (super.size() > 0)
      super.delete(0);
  }

  public void createEntry(String label, String body) {
    entryMgr.createEntry(label, body);
    append(label, null);
    this.setSelectedIndex(this.size()-1, true);
  }

  public void deleteEntry(Entry entry) {
    int index = entryMgr.getEntries().indexOf(entry);
    delete(index);
    entryMgr.deleteEntry(entry);
  }

  public void changeEntry(Entry entry) {
    int index = entryMgr.getEntries().indexOf(entry);
    set(index, entry.getLabel(), null);
    entryMgr.changeEntry(entry);
  }
  
  private Entry getSelectedEntry() {
    int index = super.getSelectedIndex();
    if (index < 0)
      return null;
    return (Entry) entryMgr.getEntries().elementAt(index);
  }
}