package win32neshto.j2mesafemod;

import java.io.IOException;
import javax.microedition.lcdui.*;

public class InitialPasswordForm extends LoginForm {
  TextField textField1;
  TextField textField2;
  J2MESafe _app;
  Command saveCommand = new Command(Str.SAVE, Command.OK, 1);
  Command restoreCommand = new Command(Str.RESTORE, Command.SCREEN, 2);

  public InitialPasswordForm(J2MESafe app) {
    super(app, Str.INIT_PASSCODE);
    this._app = app;
    append(Str.INIT_PASSCODE_DESCRIPTION);
    textField1 = new TextField(Str.ENTER_PASSCODE, "", 15, TextField.NUMERIC|TextField.PASSWORD);
    append(textField1);
    textField2 = new
            TextField(Str.RETYPE_PASSCODE, "", 15, TextField.NUMERIC|TextField.PASSWORD);
    append(textField2);
    addCommand(saveCommand);
    addCommand(restoreCommand);
  }

  public void commandAction(Command c, Displayable d) {
    if (c == saveCommand) {
      String code = textField1.getString();
      String retype = textField2.getString();
      if(!code.equals(retype)) {
          Alert a = new Alert(Str.ERROR, Str.RETYPE_NOT_EQUAL, null, AlertType.ERROR);
          a.addCommand(new Command(Str.OK, Command.OK, 0));
          Display.getDisplay(_app).setCurrent(a, this);
          return;
      }
      if(code.length()<Consts.MIN_PASSCODE_LENGTH) {
          Alert a = new Alert(Str.ERROR, Str.PASSCODE_TOO_SHORT, null, AlertType.ERROR);
          a.addCommand(new Command(Str.OK, Command.OK, 0));
          Display.getDisplay(_app).setCurrent(a, this);
          return;          
      }
      _app.passwordMgr.initialize(code);
      _app.cryptMgr.initialize(code);
      _app.entryMgr.deleteAll();
      _app.entryList.sort();
      _app.getDisplay().setCurrent(_app.entryList);
    } else if(c == restoreCommand) {
      final String code = textField1.getString();
      String retype = textField2.getString();
      if(!code.equals(retype)) {
          Alert a = new Alert(Str.ERROR, Str.RETYPE_NOT_EQUAL, null, AlertType.ERROR);
          a.addCommand(new Command(Str.OK, Command.OK, 0));
          Display.getDisplay(_app).setCurrent(a, this);
          return;
      }
      final TextBox fileNameBox = new TextBox(Str.BACKUP_FILE_NAME,
                EntryMgr.getBackupPath(), 1000, TextField.NON_PREDICTIVE);
        final Command okCommand = new Command(Str.OK, Command.OK, 1);
        final Command backCommand = new Command(Str.BACK, Command.BACK, 2);
        final Command browseCommand = new Command(Str.BROWSE, Command.SCREEN, 3);
        fileNameBox.addCommand(okCommand);
        fileNameBox.addCommand(backCommand);
        fileNameBox.addCommand(browseCommand);
        fileNameBox.setCommandListener(new CommandListener() {
                public void commandAction(Command c, Displayable d) {
                    if(c==okCommand) {
                        if(fileNameBox.getString().length()==0) {
                            return;
                        }
                        try {
                            _app.passwordMgr.initialize(code);
                            _app.cryptMgr.initialize(code);
                            boolean result = _app.entryMgr.restore(
                                    Consts.FILE_URL_PREFIX+fileNameBox.getString(), code);
                            if(!result) {
                                Alert a = new Alert(Str.ERROR, Str.INVALID_PASSCODE,
                                        null, AlertType.ERROR);
                                a.addCommand(okCommand);
                                a.setTimeout(Alert.FOREVER);
                                Display.getDisplay(_app).setCurrent(a, InitialPasswordForm.this);
                                return;
                            }
                            _app.entryList.sort();
                            Alert a = new Alert(Str.INFO, Str.SUCESSFULL_RESTORE,
                                    null, AlertType.INFO);
                            a.addCommand(okCommand);
                            Display.getDisplay(_app).setCurrent(_app.entryList);
                        } catch(Exception exc) {
                            String atext =
                                    (exc instanceof IOException)?Str.FILE_ERROR:Str.OTHER_ERROR;
                            Alert a = new Alert(Str.ERROR, atext, null, AlertType.ERROR);
                            a.addCommand(okCommand);
                            a.setTimeout(Alert.FOREVER);
                            Display.getDisplay(_app).setCurrent(a, InitialPasswordForm.this);
                        }
                    } else if(c==backCommand) {
                        Display.getDisplay(_app).setCurrent(InitialPasswordForm.this);
                    } else if(c==browseCommand) {
                        SetTextInterface sti = new SetTextInterface() {
                            public Displayable getDisplayable() {
                                return fileNameBox;
                            }

                            public void setText(String text) {
                                fileNameBox.setString(text);
                            }
                        };
                        FileBrowser fb = new FileBrowser(sti, app, false);
                        fb.show();
                    }
                }
            });
        Display.getDisplay(_app).setCurrent(fileNameBox);

    } else {
      super.commandAction(c, d);
    }
  }
}