package win32neshto.j2mesafemod;

/**
 *
 * @author Win32.Neshto
 */
public class PasswordGenerator {
    public static char[] DEFAULT_SPECIAL_CHARS = {'.', ',', '!', '?',
        '/', '-', '+', '=', '_', '@', '#', '$', '%', '^', '&', '*'};
    
    private char[] specialChars = DEFAULT_SPECIAL_CHARS;
    private int specialCharsLength = specialChars.length;

    public static final int SMALL = 1;
    public static final int CAPS = 2;
    public static final int DIGITS = 4;
    public static final int SPECIAL = 8;

    public static final int MIN_LENGTH = 5;

    private int charSet;
    private int length;
    private win32neshto.j2mesafemod.SecureRandom random;

    public PasswordGenerator(int charSet, int length) {
        if((charSet<=0)||(charSet>15)||(length<MIN_LENGTH)) {
            throw new IllegalArgumentException();
        }
        this.charSet = charSet;
        this.length = length;
        random = new win32neshto.j2mesafemod.SecureRandom();
    }
    
    public PasswordGenerator(int charSet, int length, char[] specialChars) {
        this(charSet, length);
        this.specialChars = specialChars;
        specialCharsLength = specialChars.length;
        if((specialCharsLength==0)&&((charSet&SPECIAL)!=0)) {
            throw new IllegalArgumentException();
        }
    }

    public String get() {
            boolean small = ((charSet&SMALL)!=0);
            boolean caps = ((charSet&CAPS)!=0);
            boolean digits = ((charSet&DIGITS)!=0);
            boolean special = ((charSet&SPECIAL)!=0);
            int smallCount = 0;
            int capsCount = 0;
            int digitsCount = 0;
            int specialCount = 0;
            int len1 = length;
            if(special) {
                int min = len1/5;
                int max = len1/3;
                specialCount = randomInt(min, max);
                len1 -= specialCount;
            }
            if((len1>0)&&digits) {
                int min = len1/5;
                if(min<1) {
                    min = 1;
                }
                int max = len1/3;
                if(max<1) {
                    max = 1;
                }
                digitsCount = randomInt(min, max);
                len1 -= digitsCount;
            }
            if(small&caps) {
                if(len1==1) {
                    if((random.nextInt()%2)==0) {
                        smallCount = 1;
                    } else {
                        capsCount = 1;
                    }
                } else if(len1>1) {
                    int min = 1;
                    int max = len1-1;
                    smallCount = randomInt(min, max);
                    capsCount = len1-smallCount;
                }
            } else if(small) {
                smallCount = len1;
            } else if(caps) {
                capsCount = len1;
            } else {
                if(digits&&special) {
                    if(len1==1) {
                        if((random.nextInt()%2)==0) {
                            digitsCount += 1;
                        } else {
                            specialCount += 1;
                        }
                    } else if(len1>1) {
                        int min = 1;
                        int max = len1-1;
                        int t = randomInt(min, max);
                        digitsCount += t;
                        specialCount += len1-t;
                    }
                } else if(digits) {
                    digitsCount += len1;
                } else if(special) {
                    specialCount += len1;
                }
            }
            char[] array = new char[length];
            int j = 0;
            for(int i = 0; i<smallCount; i++) {
                array[j] = randomSmall();
                j++;
            }
            for(int i = 0; i<capsCount; i++) {
                array[j] = randomCapital();
                j++;
            }
            for(int i = 0; i<digitsCount; i++) {
                array[j] = randomDigit();
                j++;
            }
            for(int i = 0; i<specialCount; i++) {
                array[j] = randomSpecial();
                j++;
            }
            shuffle(array);
            return new String(array);
    }

    private int randomInt(int min, int max) {
        if(min>max) {
            throw new IllegalArgumentException();
        } else if(min==max) {
            return min;
        } else {
            return min+Math.abs(random.nextInt())%(max-min+1);
        }
    }

    private char randomSmall() {
        int r = Math.abs(random.nextInt())%26;
        switch(r) {
            case 0: return 'a';
            case 1: return 'b';
            case 2: return 'c';
            case 3: return 'd';
            case 4: return 'e';
            case 5: return 'f';
            case 6: return 'g';
            case 7: return 'h';
            case 8: return 'i';
            case 9: return 'j';
            case 10: return 'k';
            case 11: return 'l';
            case 12: return 'm';
            case 13: return 'n';
            case 14: return 'o';
            case 15: return 'p';
            case 16: return 'q';
            case 17: return 'r';
            case 18: return 's';
            case 19: return 't';
            case 20: return 'u';
            case 21: return 'w';
            case 22: return 'v';
            case 23: return 'x';
            case 24: return 'y';
            case 25: return 'z';
            default: throw new RuntimeException();
        }
    }

    private char randomCapital() {
        int r = Math.abs(random.nextInt())%26;
        switch(r) {
            case 0: return 'A';
            case 1: return 'B';
            case 2: return 'C';
            case 3: return 'D';
            case 4: return 'E';
            case 5: return 'F';
            case 6: return 'G';
            case 7: return 'H';
            case 8: return 'I';
            case 9: return 'J';
            case 10: return 'K';
            case 11: return 'L';
            case 12: return 'M';
            case 13: return 'N';
            case 14: return 'O';
            case 15: return 'P';
            case 16: return 'Q';
            case 17: return 'R';
            case 18: return 'S';
            case 19: return 'T';
            case 20: return 'U';
            case 21: return 'W';
            case 22: return 'V';
            case 23: return 'X';
            case 24: return 'Y';
            case 25: return 'Z';
            default: throw new RuntimeException();
        }
    }

    private char randomDigit() {
        int r = Math.abs(random.nextInt())%10;
        switch(r) {
            case 0: return '0';
            case 1: return '1';
            case 2: return '2';
            case 3: return '3';
            case 4: return '4';
            case 5: return '5';
            case 6: return '6';
            case 7: return '7';
            case 8: return '8';
            case 9: return '9';
            default: throw new RuntimeException();
        }
    }

    private char randomSpecial() {
        return specialChars[Math.abs(random.nextInt())%specialCharsLength];
    }

    private void shuffle(char[] array) {
        
        int len = array.length;
        int[] weights = new int[len];
        for(int i = 0; i<len; i++) {
            weights[i] = random.nextInt();
        }
        for(int i = 0; i<len-1; i++) {
            int k = i;
            for(int j = i+1; j<len; j++) {
                if(weights[j]<weights[k]) {
                    k = j;
                }
            }
            if(i!=k) {
                char ich = array[i];
                int iw = weights[i];
                array[i] = array[k];
                weights[i] = weights[k];
                array[k] = ich;
                weights[k] = iw;
            }
        }
    }
}
