package win32neshto.j2mesafemod;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.io.*;
import javax.microedition.io.file.*;

/**
 *
 * @author Win32.Neshto
 */
public class Path {
    private String str;
    private int strlen;
    private boolean isdir;
    private static final int SEP_LEN = Consts.DIR_SEPARATOR.length();
    private static final int DSLEN = Consts.DIR_SEPARATOR.length();
    
    public Path(String str) {
        if(str.length()==0) {
            throw new IllegalArgumentException();
        }
        this.str = str;
        this.strlen = str.length();
        if(str.endsWith(Consts.DIR_SEPARATOR)) {
            isdir = true;
        } else {
            isdir = false;
        }
    }
    
    public static Path getRoot() {
        return new Path(Consts.ROOT+Consts.DIR_SEPARATOR);
    }
    
    public boolean isDirectory() {
        return isdir;
    }
    
    public String toString() {
        return str;
    }
    
    public boolean isRoot() {
        return str.equals(Consts.ROOT+Consts.DIR_SEPARATOR);
    }
    
    public Path getParentDir() {
        if(isRoot()) {
            return null;
        }
         int i = isdir?strlen-DSLEN*2:strlen-DSLEN;
        for(; i>=1; i--) {
            if(str.substring(i, i+DSLEN).equals(Consts.DIR_SEPARATOR)) {
                return new Path(str.substring(0, i+DSLEN));
            }
        }
        return Path.getRoot();
    }
    
    public String getName() {
        if(isRoot()) {
            return Consts.ROOT;
        }
        int j = isdir?strlen-DSLEN*2:strlen-DSLEN;
        for(int i = j; i>=0; i--) {
            if(str.substring(i, i+DSLEN).equals(Consts.DIR_SEPARATOR)) {
                return str.substring(i+DSLEN, isdir?strlen-DSLEN:strlen);
            }
        }
        return str.substring(0, j);
    }
    
    public void list(Vector dirs, Vector files) throws IOException {
        if(!isdir) {
            throw new IOException();
        }
        dirs.removeAllElements();
        files.removeAllElements();
        if(isRoot()) {
            Enumeration e = FileSystemRegistry.listRoots();
            while(e.hasMoreElements()) {
                String name = (String)e.nextElement();
                dirs.addElement(new Path(Consts.ROOT+name));
            }
        } else {
            FileConnection fc = (FileConnection)Connector.open(Consts.FILE_URL_PREFIX+str);
            try {
                Enumeration e = fc.list();
                while(e.hasMoreElements()) {
                    String name = (String)e.nextElement();
                    if(name.endsWith(Consts.DIR_SEPARATOR)) {
                        dirs.addElement(new Path(str+name));
                    } else {
                        files.addElement(new Path(str+name));
                    }
                }
            } finally {
                fc.close();
            }
        }
        sortPathVector(dirs);
        sortPathVector(files);
    }
    
    public static void sortPathVector(Vector v) {
        int l = v.size();
        for(int i = 0; i<l-1; i++) {
            int k = i;
            Path si = (Path)v.elementAt(i);
            Path sk = (Path)v.elementAt(k);
            for(int j = 0; j<l; j++) {
                Path sj = (Path)v.elementAt(j);
                if(sj.getName().compareTo(sk.getName())<0) {
                    k = j;
                    sk = sj;
                }
            }
            if(k!=i) {
                v.setElementAt(sk, i);
                v.setElementAt(si, k);
            }
        }
    }
}
